#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>

// PIN Settings
#define Bewegungsmelder 2
#define OnlinestatusLED 4
#define Schaltrelais 5

// Variable Declaration
int Programmfrequenz = 50;  // loop-Durchlauf pro Sekunde (ca.)

int HelligkeitssensordIdleSek = 5; // Minimale Pause zwischen zwei Helligkeitbenachrichtigungen in Sek
int HelligkeitssensorSchwellwert = 1; // Schwellwert fuer Helligkeitsbenachrichtigungen (empfohlen 5-20)

int OnlinestatusCheckEachSekOnline = 30; // Ping in Sek wenn letzter Status Online war
int OnlinestatusCheckEachSekOffline = 5; // Ping in Sek wenn letzter Status Offline war
int OnlinestatusCheckTimeoutSek = 3; // Status auf Offline setzen wenn Rueckmeldung nicht innerhalb x Sek

char BewegungsmelderState = 0;
int Helligkeitssensor = 0;
int HelligkeitssensorCounter = 0;
int HelligkeitssensorObergrenze = 0;
int HelligkeitssensorUntergrenze = 0;
char msg[25];
int HelligkeitssensordIdle = HelligkeitssensordIdleSek * Programmfrequenz;
int OnlinestatusSendCounter = 0;
int OnlinestatusReceiveCounter = 0;
char OnlinestatusLoxone = 0;
int OnlinestatusCheckOnline = OnlinestatusCheckEachSekOnline * Programmfrequenz;
int OnlinestatusCheckOffline = OnlinestatusCheckEachSekOffline * Programmfrequenz;
int OnlinestatusCheckTimeout = (OnlinestatusCheckTimeoutSek + OnlinestatusCheckEachSekOnline) * Programmfrequenz;

// Network Settings
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 3, 19);
IPAddress gateway(192, 168, 3, 1);
IPAddress subnet(255, 255, 255, 0);

// Local UDP port to listen on
unsigned int localPort = 7002;

// Recipient IP
IPAddress RecipientIP(192, 168, 3, 11);

// Recipient UDP Port
unsigned int RecipientPort = 7001;

// buffers for receiving and sending data
char packetBuffer[UDP_TX_PACKET_MAX_SIZE];

// An EthernetUDP instance to send and receive packets over UDP
EthernetUDP Udp;

void setup()
{
  // for debug only
  Serial.begin(9600);

  // start Ethernet
  Ethernet.begin(mac,ip);

  // Start UDP
  Udp.begin(localPort);

  // Bewegungsmelder 
  pinMode(Bewegungsmelder, INPUT);

  //OnlinestatusLED
  pinMode(OnlinestatusLED, OUTPUT);   // LED an Pin4

  //Schaltrelais
  pinMode(Schaltrelais, OUTPUT);   // Schaltrelais an Pin5

  // send UDP Ready 
  sendUDP("UDP Ready");
}

void loop()
{

  checkUDP();

  // Bewegungsmelder (Send)
  if (digitalRead(Bewegungsmelder) == LOW && BewegungsmelderState != 0)
  {
    sendUDP("WZ.Bewegungsmelder: 0");
    BewegungsmelderState = 0;
  }

  if (digitalRead(Bewegungsmelder) == HIGH && BewegungsmelderState != 1)
  {
    sendUDP("WZ.Bewegungsmelder: 1");
    BewegungsmelderState = 1;
  }

  // Helligkeitssensor (Send)
  Helligkeitssensor=(analogRead(A0)-100)/8;

  HelligkeitssensorCounter = HelligkeitssensorCounter + 1;

  if (Helligkeitssensor >= HelligkeitssensorObergrenze || Helligkeitssensor <= HelligkeitssensorUntergrenze)
  {

    if (HelligkeitssensorCounter >= HelligkeitssensordIdle)
    {
      sprintf(msg, "WZ.Helligkeitssensor: %d", Helligkeitssensor);
      Serial.println(msg);
      sendUDP(msg);

      HelligkeitssensorObergrenze = Helligkeitssensor + HelligkeitssensorSchwellwert;
      HelligkeitssensorUntergrenze = Helligkeitssensor - HelligkeitssensorSchwellwert;
      HelligkeitssensorCounter = 0;
    }
  }

  // Onlinestatus-Ping (Send)
  OnlinestatusSendCounter = OnlinestatusSendCounter + 1;

  if (OnlinestatusSendCounter == OnlinestatusCheckOnline && OnlinestatusReceiveCounter != OnlinestatusCheckTimeout)
  {
    Serial.println("SendUDP: Ping");
    sendUDP("Ping");
    OnlinestatusSendCounter = 0;
  }

  if (OnlinestatusSendCounter == OnlinestatusCheckOffline && OnlinestatusReceiveCounter == OnlinestatusCheckTimeout)
  {
    Serial.println("SendUDP: Ping");
    sendUDP("Ping");
    OnlinestatusSendCounter = 0;
  }

  // Onlinestatus-Pong (Receive)
  if (!strcmp(packetBuffer, "000"))
  {
    OnlinestatusReceiveCounter = 0;
  }

  if (!strcmp(packetBuffer, "001"))
  {
    OnlinestatusLoxone = 1;
    digitalWrite(OnlinestatusLED, HIGH);
  }

  if (OnlinestatusReceiveCounter < OnlinestatusCheckTimeout)
  {
    OnlinestatusReceiveCounter = OnlinestatusReceiveCounter + 1;
  }

  if (OnlinestatusReceiveCounter == OnlinestatusCheckTimeout)
  {
    OnlinestatusLoxone = 0;
    digitalWrite(OnlinestatusLED, LOW);
  }

  // Schaltrelais
  if (!strcmp(packetBuffer, "051"))
  {
    digitalWrite(Schaltrelais, HIGH);
  }

  if (!strcmp(packetBuffer, "050"))
  {
    digitalWrite(Schaltrelais, LOW);
  }

delay(1000/Programmfrequenz);
}

// Function to send UDP packets
void sendUDP(String text)
{
    Udp.beginPacket(RecipientIP, RecipientPort);
    // Udp.write("Test");
    Udp.print(text);
    Udp.endPacket();
}

// Function to check for new incoming UDP packets
void checkUDP()
{  
  // if there's data available, read a packet
  int packetSize = Udp.parsePacket();
  if(packetSize)
  {
    // read the packet into packetBufffer
    Udp.read(packetBuffer,UDP_TX_PACKET_MAX_SIZE);
    
    // For debug only
    // Write packetBuffer to serial
    Serial.print("ReceiveUDP: ");
    Serial.println(packetBuffer);
    
    // send "ACK" as reply
    //Udp.beginPacket(Udp.remoteIP(), Udp.remotePort());
    //Udp.write("ACK");
    //Udp.endPacket();
  }
}
